# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import (
    product_list_products_params,
    product_create_product_params,
    product_update_product_params,
    product_duplicate_product_params,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.product import Product
from ...types.v1.product_list_products_response import ProductListProductsResponse

__all__ = ["ProductsResource", "AsyncProductsResource"]


class ProductsResource(SyncAPIResource):
    """Operations related to products"""

    @cached_property
    def with_raw_response(self) -> ProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return ProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return ProductsResourceWithStreamingResponse(self)

    def archive_product(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """Archives a product, preventing new subscriptions.

        All plans and addons are
        archived.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/products/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def create_product(
        self,
        *,
        id: str,
        display_name: str,
        description: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        multiple_subscriptions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Creates a new product.

        Args:
          id: The unique identifier for the entity

          display_name: Display name of the product

          description: Description of the product

          metadata: Additional metadata for the product

          multiple_subscriptions: Indicates if multiple subscriptions to this product are allowed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/products",
            body=maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "description": description,
                    "metadata": metadata,
                    "multiple_subscriptions": multiple_subscriptions,
                },
                product_create_product_params.ProductCreateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def duplicate_product(
        self,
        path_id: str,
        *,
        body_id: str,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Duplicates an existing product, including its plans, addons, and configuration.

        Args:
          body_id: The unique identifier for the entity

          description: Description of the product

          display_name: Display name of the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        return self._post(
            f"/api/v1/products/{path_id}/duplicate",
            body=maybe_transform(
                {
                    "body_id": body_id,
                    "description": description,
                    "display_name": display_name,
                },
                product_duplicate_product_params.ProductDuplicateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def list_products(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: product_list_products_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[ProductListProductsResponse]:
        """
        Retrieves a paginated list of products in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by product status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/products",
            page=SyncMyCursorIDPage[ProductListProductsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                    },
                    product_list_products_params.ProductListProductsParams,
                ),
            ),
            model=ProductListProductsResponse,
        )

    def unarchive_product(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Restores an archived product, allowing new subscriptions to be created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def update_product(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        multiple_subscriptions: bool | Omit = omit,
        product_settings: product_update_product_params.ProductSettings | Omit = omit,
        usage_reset_cutoff_rule: product_update_product_params.UsageResetCutoffRule | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Updates an existing product's properties such as display name, description, and
        metadata.

        Args:
          description: Description of the product

          display_name: Display name of the product

          metadata: Additional metadata for the product

          multiple_subscriptions: Indicates if multiple subscriptions to this product are allowed

          usage_reset_cutoff_rule: Rule defining when usage resets upon subscription update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/products/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "display_name": display_name,
                    "metadata": metadata,
                    "multiple_subscriptions": multiple_subscriptions,
                    "product_settings": product_settings,
                    "usage_reset_cutoff_rule": usage_reset_cutoff_rule,
                },
                product_update_product_params.ProductUpdateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )


class AsyncProductsResource(AsyncAPIResource):
    """Operations related to products"""

    @cached_property
    def with_raw_response(self) -> AsyncProductsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProductsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProductsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncProductsResourceWithStreamingResponse(self)

    async def archive_product(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """Archives a product, preventing new subscriptions.

        All plans and addons are
        archived.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/products/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def create_product(
        self,
        *,
        id: str,
        display_name: str,
        description: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        multiple_subscriptions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Creates a new product.

        Args:
          id: The unique identifier for the entity

          display_name: Display name of the product

          description: Description of the product

          metadata: Additional metadata for the product

          multiple_subscriptions: Indicates if multiple subscriptions to this product are allowed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/products",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "description": description,
                    "metadata": metadata,
                    "multiple_subscriptions": multiple_subscriptions,
                },
                product_create_product_params.ProductCreateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def duplicate_product(
        self,
        path_id: str,
        *,
        body_id: str,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Duplicates an existing product, including its plans, addons, and configuration.

        Args:
          body_id: The unique identifier for the entity

          description: Description of the product

          display_name: Display name of the product

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_id:
            raise ValueError(f"Expected a non-empty value for `path_id` but received {path_id!r}")
        return await self._post(
            f"/api/v1/products/{path_id}/duplicate",
            body=await async_maybe_transform(
                {
                    "body_id": body_id,
                    "description": description,
                    "display_name": display_name,
                },
                product_duplicate_product_params.ProductDuplicateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    def list_products(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: product_list_products_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProductListProductsResponse, AsyncMyCursorIDPage[ProductListProductsResponse]]:
        """
        Retrieves a paginated list of products in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by product status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/products",
            page=AsyncMyCursorIDPage[ProductListProductsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                    },
                    product_list_products_params.ProductListProductsParams,
                ),
            ),
            model=ProductListProductsResponse,
        )

    async def unarchive_product(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Restores an archived product, allowing new subscriptions to be created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/products/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )

    async def update_product(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        multiple_subscriptions: bool | Omit = omit,
        product_settings: product_update_product_params.ProductSettings | Omit = omit,
        usage_reset_cutoff_rule: product_update_product_params.UsageResetCutoffRule | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Product:
        """
        Updates an existing product's properties such as display name, description, and
        metadata.

        Args:
          description: Description of the product

          display_name: Display name of the product

          metadata: Additional metadata for the product

          multiple_subscriptions: Indicates if multiple subscriptions to this product are allowed

          usage_reset_cutoff_rule: Rule defining when usage resets upon subscription update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/products/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "display_name": display_name,
                    "metadata": metadata,
                    "multiple_subscriptions": multiple_subscriptions,
                    "product_settings": product_settings,
                    "usage_reset_cutoff_rule": usage_reset_cutoff_rule,
                },
                product_update_product_params.ProductUpdateProductParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Product,
        )


class ProductsResourceWithRawResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.archive_product = to_raw_response_wrapper(
            products.archive_product,
        )
        self.create_product = to_raw_response_wrapper(
            products.create_product,
        )
        self.duplicate_product = to_raw_response_wrapper(
            products.duplicate_product,
        )
        self.list_products = to_raw_response_wrapper(
            products.list_products,
        )
        self.unarchive_product = to_raw_response_wrapper(
            products.unarchive_product,
        )
        self.update_product = to_raw_response_wrapper(
            products.update_product,
        )


class AsyncProductsResourceWithRawResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.archive_product = async_to_raw_response_wrapper(
            products.archive_product,
        )
        self.create_product = async_to_raw_response_wrapper(
            products.create_product,
        )
        self.duplicate_product = async_to_raw_response_wrapper(
            products.duplicate_product,
        )
        self.list_products = async_to_raw_response_wrapper(
            products.list_products,
        )
        self.unarchive_product = async_to_raw_response_wrapper(
            products.unarchive_product,
        )
        self.update_product = async_to_raw_response_wrapper(
            products.update_product,
        )


class ProductsResourceWithStreamingResponse:
    def __init__(self, products: ProductsResource) -> None:
        self._products = products

        self.archive_product = to_streamed_response_wrapper(
            products.archive_product,
        )
        self.create_product = to_streamed_response_wrapper(
            products.create_product,
        )
        self.duplicate_product = to_streamed_response_wrapper(
            products.duplicate_product,
        )
        self.list_products = to_streamed_response_wrapper(
            products.list_products,
        )
        self.unarchive_product = to_streamed_response_wrapper(
            products.unarchive_product,
        )
        self.update_product = to_streamed_response_wrapper(
            products.update_product,
        )


class AsyncProductsResourceWithStreamingResponse:
    def __init__(self, products: AsyncProductsResource) -> None:
        self._products = products

        self.archive_product = async_to_streamed_response_wrapper(
            products.archive_product,
        )
        self.create_product = async_to_streamed_response_wrapper(
            products.create_product,
        )
        self.duplicate_product = async_to_streamed_response_wrapper(
            products.duplicate_product,
        )
        self.list_products = async_to_streamed_response_wrapper(
            products.list_products,
        )
        self.unarchive_product = async_to_streamed_response_wrapper(
            products.unarchive_product,
        )
        self.update_product = async_to_streamed_response_wrapper(
            products.update_product,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1.subscriptions import UsageSyncResponse, UsageChargeUsageResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsage:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_charge_usage(self, client: Stigg) -> None:
        usage = client.v1.subscriptions.usage.charge_usage(
            id="x",
        )
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_charge_usage_with_all_params(self, client: Stigg) -> None:
        usage = client.v1.subscriptions.usage.charge_usage(
            id="x",
            until_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_charge_usage(self, client: Stigg) -> None:
        response = client.v1.subscriptions.usage.with_raw_response.charge_usage(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_charge_usage(self, client: Stigg) -> None:
        with client.v1.subscriptions.usage.with_streaming_response.charge_usage(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_charge_usage(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.usage.with_raw_response.charge_usage(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_sync(self, client: Stigg) -> None:
        usage = client.v1.subscriptions.usage.sync(
            "x",
        )
        assert_matches_type(UsageSyncResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_sync(self, client: Stigg) -> None:
        response = client.v1.subscriptions.usage.with_raw_response.sync(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageSyncResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_sync(self, client: Stigg) -> None:
        with client.v1.subscriptions.usage.with_streaming_response.sync(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageSyncResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_sync(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.usage.with_raw_response.sync(
                "",
            )


class TestAsyncUsage:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_charge_usage(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.subscriptions.usage.charge_usage(
            id="x",
        )
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_charge_usage_with_all_params(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.subscriptions.usage.charge_usage(
            id="x",
            until_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_charge_usage(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.usage.with_raw_response.charge_usage(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_charge_usage(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.usage.with_streaming_response.charge_usage(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageChargeUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_charge_usage(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.usage.with_raw_response.charge_usage(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_sync(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.subscriptions.usage.sync(
            "x",
        )
        assert_matches_type(UsageSyncResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_sync(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.usage.with_raw_response.sync(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageSyncResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_sync(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.usage.with_streaming_response.sync(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageSyncResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_sync(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.usage.with_raw_response.sync(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .plan_entitlement import PlanEntitlement as PlanEntitlement
from .entitlement_create_params import EntitlementCreateParams as EntitlementCreateParams
from .entitlement_list_response import EntitlementListResponse as EntitlementListResponse
from .entitlement_update_params import EntitlementUpdateParams as EntitlementUpdateParams
from .entitlement_create_response import EntitlementCreateResponse as EntitlementCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import (
    CustomerResponse,
    CustomerListResponse,
    CustomerImportResponse,
    CustomerListResourcesResponse,
)
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCustomers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Stigg) -> None:
        customer = client.v1.customers.retrieve(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Stigg) -> None:
        customer = client.v1.customers.update(
            id="x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Stigg) -> None:
        customer = client.v1.customers.update(
            id="x",
            billing_currency="usd",
            billing_id="billingId",
            coupon_id="couponId",
            email="dev@stainless.com",
            integrations=[
                {
                    "id": "id",
                    "synced_entity_id": "syncedEntityId",
                    "vendor_identifier": "AUTH0",
                }
            ],
            language="language",
            metadata={"foo": "string"},
            name="name",
            passthrough={
                "stripe": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "customer_name": "customerName",
                    "invoice_custom_fields": {"foo": "string"},
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "shipping_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "tax_ids": [
                        {
                            "type": "type",
                            "value": "value",
                        }
                    ],
                },
                "zuora": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "currency": "usd",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                },
            },
            timezone="timezone",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        customer = client.v1.customers.list()
        assert_matches_type(SyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Stigg) -> None:
        customer = client.v1.customers.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            email="email",
            limit=1,
            name="name",
        )
        assert_matches_type(SyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(SyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(SyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive(self, client: Stigg) -> None:
        customer = client.v1.customers.archive(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_archive(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.archive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_archive(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.archive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_archive(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_import(self, client: Stigg) -> None:
        customer = client.v1.customers.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        )
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_import_with_all_params(self, client: Stigg) -> None:
        customer = client.v1.customers.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                    "billing_id": "billingId",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "salesforce_id": "salesforceId",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            integration_id="integrationId",
        )
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_import(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_import(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerImportResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_resources(self, client: Stigg) -> None:
        customer = client.v1.customers.list_resources(
            id="x",
        )
        assert_matches_type(SyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_resources_with_all_params(self, client: Stigg) -> None:
        customer = client.v1.customers.list_resources(
            id="x",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=1,
        )
        assert_matches_type(SyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_resources(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.list_resources(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(SyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_resources(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.list_resources(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(SyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_resources(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.with_raw_response.list_resources(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_provision(self, client: Stigg) -> None:
        customer = client.v1.customers.provision(
            id="id",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_provision_with_all_params(self, client: Stigg) -> None:
        customer = client.v1.customers.provision(
            id="id",
            billing_currency="usd",
            billing_id="billingId",
            coupon_id="couponId",
            default_payment_method={
                "billing_id": "billingId",
                "card_expiry_month": 0,
                "card_expiry_year": 0,
                "card_last4_digits": "cardLast4Digits",
                "type": "CARD",
            },
            email="dev@stainless.com",
            integrations=[
                {
                    "id": "id",
                    "synced_entity_id": "syncedEntityId",
                    "vendor_identifier": "AUTH0",
                }
            ],
            language="language",
            metadata={"foo": "string"},
            name="name",
            passthrough={
                "stripe": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "customer_name": "customerName",
                    "invoice_custom_fields": {"foo": "string"},
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "shipping_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "tax_ids": [
                        {
                            "type": "type",
                            "value": "value",
                        }
                    ],
                },
                "zuora": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "currency": "usd",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                },
            },
            timezone="timezone",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_provision(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.provision(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_provision(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.provision(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_unarchive(self, client: Stigg) -> None:
        customer = client.v1.customers.unarchive(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_unarchive(self, client: Stigg) -> None:
        response = client.v1.customers.with_raw_response.unarchive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_unarchive(self, client: Stigg) -> None:
        with client.v1.customers.with_streaming_response.unarchive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_unarchive(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.customers.with_raw_response.unarchive(
                "",
            )


class TestAsyncCustomers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.retrieve(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.update(
            id="x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.update(
            id="x",
            billing_currency="usd",
            billing_id="billingId",
            coupon_id="couponId",
            email="dev@stainless.com",
            integrations=[
                {
                    "id": "id",
                    "synced_entity_id": "syncedEntityId",
                    "vendor_identifier": "AUTH0",
                }
            ],
            language="language",
            metadata={"foo": "string"},
            name="name",
            passthrough={
                "stripe": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "customer_name": "customerName",
                    "invoice_custom_fields": {"foo": "string"},
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "shipping_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "tax_ids": [
                        {
                            "type": "type",
                            "value": "value",
                        }
                    ],
                },
                "zuora": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "currency": "usd",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                },
            },
            timezone="timezone",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.list()
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            email="email",
            limit=1,
            name="name",
        )
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[CustomerListResponse], customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.archive(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.archive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.archive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_archive(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_import(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        )
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_import_with_all_params(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                    "billing_id": "billingId",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "salesforce_id": "salesforceId",
                    "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            integration_id="integrationId",
        )
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_import(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerImportResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_import(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.import_(
            customers=[
                {
                    "id": "id",
                    "email": "dev@stainless.com",
                    "name": "name",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerImportResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_resources(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.list_resources(
            id="x",
        )
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_resources_with_all_params(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.list_resources(
            id="x",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=1,
        )
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_resources(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.list_resources(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_resources(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.list_resources(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[CustomerListResourcesResponse], customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_resources(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.with_raw_response.list_resources(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_provision(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.provision(
            id="id",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_provision_with_all_params(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.provision(
            id="id",
            billing_currency="usd",
            billing_id="billingId",
            coupon_id="couponId",
            default_payment_method={
                "billing_id": "billingId",
                "card_expiry_month": 0,
                "card_expiry_year": 0,
                "card_last4_digits": "cardLast4Digits",
                "type": "CARD",
            },
            email="dev@stainless.com",
            integrations=[
                {
                    "id": "id",
                    "synced_entity_id": "syncedEntityId",
                    "vendor_identifier": "AUTH0",
                }
            ],
            language="language",
            metadata={"foo": "string"},
            name="name",
            passthrough={
                "stripe": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "customer_name": "customerName",
                    "invoice_custom_fields": {"foo": "string"},
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                    "shipping_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "tax_ids": [
                        {
                            "type": "type",
                            "value": "value",
                        }
                    ],
                },
                "zuora": {
                    "billing_address": {
                        "city": "city",
                        "country": "country",
                        "line1": "line1",
                        "line2": "line2",
                        "postal_code": "postalCode",
                        "state": "state",
                    },
                    "currency": "usd",
                    "metadata": {"foo": "string"},
                    "payment_method_id": "paymentMethodId",
                },
            },
            timezone="timezone",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_provision(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.provision(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_provision(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.provision(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_unarchive(self, async_client: AsyncStigg) -> None:
        customer = await async_client.v1.customers.unarchive(
            "x",
        )
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_unarchive(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.customers.with_raw_response.unarchive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerResponse, customer, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_unarchive(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.customers.with_streaming_response.unarchive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_unarchive(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.customers.with_raw_response.unarchive(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .grants import (
    GrantsResource,
    AsyncGrantsResource,
    GrantsResourceWithRawResponse,
    AsyncGrantsResourceWithRawResponse,
    GrantsResourceWithStreamingResponse,
    AsyncGrantsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.v1.events import credit_get_usage_params, credit_list_ledger_params, credit_get_auto_recharge_params
from .....types.v1.events.credit_get_usage_response import CreditGetUsageResponse
from .....types.v1.events.credit_list_ledger_response import CreditListLedgerResponse
from .....types.v1.events.credit_get_auto_recharge_response import CreditGetAutoRechargeResponse

__all__ = ["CreditsResource", "AsyncCreditsResource"]


class CreditsResource(SyncAPIResource):
    @cached_property
    def grants(self) -> GrantsResource:
        """Operations related to credit grants"""
        return GrantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return CreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return CreditsResourceWithStreamingResponse(self)

    def get_auto_recharge(
        self,
        *,
        currency_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetAutoRechargeResponse:
        """
        Retrieves the automatic recharge configuration for a customer and currency.
        Returns default settings if no configuration exists.

        Args:
          currency_id: Filter by currency ID (required)

          customer_id: Filter by customer ID (required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/credits/auto-recharge",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "currency_id": currency_id,
                        "customer_id": customer_id,
                    },
                    credit_get_auto_recharge_params.CreditGetAutoRechargeParams,
                ),
            ),
            cast_to=CreditGetAutoRechargeResponse,
        )

    def get_usage(
        self,
        *,
        customer_id: str,
        currency_id: str | Omit = omit,
        resource_id: str | Omit = omit,
        time_range: Literal["LAST_DAY", "LAST_WEEK", "LAST_MONTH", "LAST_YEAR"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetUsageResponse:
        """
        Retrieves credit usage time-series data for a customer, grouped by feature, over
        a specified time range.

        Args:
          customer_id: Filter by customer ID (required)

          currency_id: Filter by currency ID

          resource_id: Filter by resource ID

          time_range: Time range for usage data (LAST_DAY, LAST_WEEK, LAST_MONTH, LAST_YEAR). Defaults
              to LAST_MONTH

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/credits/usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "currency_id": currency_id,
                        "resource_id": resource_id,
                        "time_range": time_range,
                    },
                    credit_get_usage_params.CreditGetUsageParams,
                ),
            ),
            cast_to=CreditGetUsageResponse,
        )

    def list_ledger(
        self,
        *,
        customer_id: str,
        after: str | Omit = omit,
        before: str | Omit = omit,
        currency_id: str | Omit = omit,
        limit: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[CreditListLedgerResponse]:
        """
        Retrieves a paginated list of credit ledger events for a customer.

        Args:
          customer_id: Filter by customer ID (required)

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          currency_id: Filter by currency ID

          limit: Maximum number of items to return

          resource_id: Filter by resource ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/credits/ledger",
            page=SyncMyCursorIDPage[CreditListLedgerResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "after": after,
                        "before": before,
                        "currency_id": currency_id,
                        "limit": limit,
                        "resource_id": resource_id,
                    },
                    credit_list_ledger_params.CreditListLedgerParams,
                ),
            ),
            model=CreditListLedgerResponse,
        )


class AsyncCreditsResource(AsyncAPIResource):
    @cached_property
    def grants(self) -> AsyncGrantsResource:
        """Operations related to credit grants"""
        return AsyncGrantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncCreditsResourceWithStreamingResponse(self)

    async def get_auto_recharge(
        self,
        *,
        currency_id: str,
        customer_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetAutoRechargeResponse:
        """
        Retrieves the automatic recharge configuration for a customer and currency.
        Returns default settings if no configuration exists.

        Args:
          currency_id: Filter by currency ID (required)

          customer_id: Filter by customer ID (required)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/credits/auto-recharge",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "currency_id": currency_id,
                        "customer_id": customer_id,
                    },
                    credit_get_auto_recharge_params.CreditGetAutoRechargeParams,
                ),
            ),
            cast_to=CreditGetAutoRechargeResponse,
        )

    async def get_usage(
        self,
        *,
        customer_id: str,
        currency_id: str | Omit = omit,
        resource_id: str | Omit = omit,
        time_range: Literal["LAST_DAY", "LAST_WEEK", "LAST_MONTH", "LAST_YEAR"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetUsageResponse:
        """
        Retrieves credit usage time-series data for a customer, grouped by feature, over
        a specified time range.

        Args:
          customer_id: Filter by customer ID (required)

          currency_id: Filter by currency ID

          resource_id: Filter by resource ID

          time_range: Time range for usage data (LAST_DAY, LAST_WEEK, LAST_MONTH, LAST_YEAR). Defaults
              to LAST_MONTH

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/credits/usage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "customer_id": customer_id,
                        "currency_id": currency_id,
                        "resource_id": resource_id,
                        "time_range": time_range,
                    },
                    credit_get_usage_params.CreditGetUsageParams,
                ),
            ),
            cast_to=CreditGetUsageResponse,
        )

    def list_ledger(
        self,
        *,
        customer_id: str,
        after: str | Omit = omit,
        before: str | Omit = omit,
        currency_id: str | Omit = omit,
        limit: int | Omit = omit,
        resource_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CreditListLedgerResponse, AsyncMyCursorIDPage[CreditListLedgerResponse]]:
        """
        Retrieves a paginated list of credit ledger events for a customer.

        Args:
          customer_id: Filter by customer ID (required)

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          currency_id: Filter by currency ID

          limit: Maximum number of items to return

          resource_id: Filter by resource ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/credits/ledger",
            page=AsyncMyCursorIDPage[CreditListLedgerResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "after": after,
                        "before": before,
                        "currency_id": currency_id,
                        "limit": limit,
                        "resource_id": resource_id,
                    },
                    credit_list_ledger_params.CreditListLedgerParams,
                ),
            ),
            model=CreditListLedgerResponse,
        )


class CreditsResourceWithRawResponse:
    def __init__(self, credits: CreditsResource) -> None:
        self._credits = credits

        self.get_auto_recharge = to_raw_response_wrapper(
            credits.get_auto_recharge,
        )
        self.get_usage = to_raw_response_wrapper(
            credits.get_usage,
        )
        self.list_ledger = to_raw_response_wrapper(
            credits.list_ledger,
        )

    @cached_property
    def grants(self) -> GrantsResourceWithRawResponse:
        """Operations related to credit grants"""
        return GrantsResourceWithRawResponse(self._credits.grants)


class AsyncCreditsResourceWithRawResponse:
    def __init__(self, credits: AsyncCreditsResource) -> None:
        self._credits = credits

        self.get_auto_recharge = async_to_raw_response_wrapper(
            credits.get_auto_recharge,
        )
        self.get_usage = async_to_raw_response_wrapper(
            credits.get_usage,
        )
        self.list_ledger = async_to_raw_response_wrapper(
            credits.list_ledger,
        )

    @cached_property
    def grants(self) -> AsyncGrantsResourceWithRawResponse:
        """Operations related to credit grants"""
        return AsyncGrantsResourceWithRawResponse(self._credits.grants)


class CreditsResourceWithStreamingResponse:
    def __init__(self, credits: CreditsResource) -> None:
        self._credits = credits

        self.get_auto_recharge = to_streamed_response_wrapper(
            credits.get_auto_recharge,
        )
        self.get_usage = to_streamed_response_wrapper(
            credits.get_usage,
        )
        self.list_ledger = to_streamed_response_wrapper(
            credits.list_ledger,
        )

    @cached_property
    def grants(self) -> GrantsResourceWithStreamingResponse:
        """Operations related to credit grants"""
        return GrantsResourceWithStreamingResponse(self._credits.grants)


class AsyncCreditsResourceWithStreamingResponse:
    def __init__(self, credits: AsyncCreditsResource) -> None:
        self._credits = credits

        self.get_auto_recharge = async_to_streamed_response_wrapper(
            credits.get_auto_recharge,
        )
        self.get_usage = async_to_streamed_response_wrapper(
            credits.get_usage,
        )
        self.list_ledger = async_to_streamed_response_wrapper(
            credits.list_ledger,
        )

    @cached_property
    def grants(self) -> AsyncGrantsResourceWithStreamingResponse:
        """Operations related to credit grants"""
        return AsyncGrantsResourceWithStreamingResponse(self._credits.grants)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .credits import (
    CreditsResource,
    AsyncCreditsResource,
    CreditsResourceWithRawResponse,
    AsyncCreditsResourceWithRawResponse,
    CreditsResourceWithStreamingResponse,
    AsyncCreditsResourceWithStreamingResponse,
)

__all__ = [
    "CreditsResource",
    "AsyncCreditsResource",
    "CreditsResourceWithRawResponse",
    "AsyncCreditsResourceWithRawResponse",
    "CreditsResourceWithStreamingResponse",
    "AsyncCreditsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
]
